from graph import LinkedDirectedGraph
from graph import LinkedVertex

class LinkedVertexPrevious(LinkedVertex):
    """ Augment LinkedVertex to include a _previous attribute to
        track which vertex was visited immediately before this one"""

    def __init__(self, label):
        LinkedVertex.__init__(self, label)
        self._previous = None

    def getPrevious(self): 
        return self._previous
    
    def hasPrevious(self): 
        return self._previous != None
    
    def setPrevious(self, previous):
        self._previous = previous
        
    def __str__(self):
        return 'Label='+ str(self._label) + " Previous="+str(self._previous)

class LinkedDirectedGraphPrevious(LinkedDirectedGraph):
    """ Augment LinkedDirectedGraph to manipulate the _previous attributes"""
   
    def clearVertexPrevious(self):
        for vertex in self.vertices():
            vertex.setPrevious(None)

    def traversePrevious(self, end):
        """ Returns the list of labels from traversing the _previous attributes
            from 'end' backward until None reached""" 
        lyst = []
        while end != None:
            lyst.append(end)
            end = self.getVertex(end).getPrevious()
        return lyst
        
    def addVertex(self, label):
        self._vertices[label] = LinkedVertexPrevious(label)
        self._vertexCount += 1
  
